'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createObjectMethodTransform;

var _package = require('../../package.json');

var _createImport = require('./createImport');

var _createImport2 = _interopRequireDefault(_createImport);

var _isPolyfillPath = require('./isPolyfillPath');

var _isPolyfillPath2 = _interopRequireDefault(_isPolyfillPath);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function createImportIdentifier(file, polyfill) {
  return (0, _createImport2.default)(file, polyfill.split('.').join('_'), `${_package.name}/polyfills/${polyfill.split('.').join('/')}`);
}

function createObjectMethodTransform(polyfill) {
  const [object, property] = polyfill.split('.');

  return {
    MemberExpression(path, { file }, { types: t }) {
      const { filename } = file.opts;

      // Skip when already polyfilled
      if ((0, _isPolyfillPath2.default)(filename)) {
        return;
      }

      // Guard against invalid types
      if (t.isIdentifier(path.node)) {
        return;
      }

      // Replace with polyfill
      if (path.node.object.name === object && path.node.property.name === property) {
        path.replaceWith(createImportIdentifier(file, polyfill));
      }
    },

    CallExpression(path, { file }, { types: t }) {
      const { filename } = file.opts;

      // Skip when already polyfilled
      if ((0, _isPolyfillPath2.default)(filename)) {
        return;
      }

      // Guard against invalid types
      if (!t.isMemberExpression(path.node.callee) || !t.isIdentifier(path.node.callee.object) || !t.isIdentifier(path.node.callee.property)) {
        return;
      }

      // Replace with polyfill
      if (path.node.callee.object.name === object && path.node.callee.property.name === property) {
        path.replaceWith(t.callExpression(createImportIdentifier(file, polyfill), path.node.arguments));
      }
    }
  };
}